##########################################################
## 
## GAM example: mstop controls smoothness of splines
## 
###########################################################
##
## Authors of the Code: Andreas Mayr, Benjamin Hofner
## 
## This code accompanies the paper
##
##   Boosting for statistical modelling -- 
##   a non-technical introduction
## 
##   by Andreas Mayr and Benjamin Hofner
## 
###########################################################


## Set seed to make analysis reproducible
set.seed(1234)

###########################################################
## Install package if not available
###########################################################

## Install the actual boosting packae
if (!require("mboost")) {
  install.packages("mboost")
}

###########################################################
## Load required package
###########################################################

library("mboost")

###########################################################
## Simulate data 
###########################################################

## uniformly distributed covariate
x <- runif(150, -0.2, 0.2)

## outcome following a pre-defined curve + noise
y <- (0.5 - 0.9* exp(-50*x^2))*x + 0.02 *rnorm(150)

y <- y[order(x)] ## order obs by size of x
x <- x[order(x)] ## just for easier plotting

###########################################################
## Plotting
###########################################################


pdf("Figures/gam_show.pdf", width = 7.5, height = 10)
par(mfrow = c(4,3), mar = c(2, 3, 2.5, 0.5), cex.main = 1.3)
plot(x, y, las = 1, main = "m = 0", pch = 19, xlab = "", ylab = "", 
     col = rgb(0.7, 0.7, 0.7, 0.75)) ## observations
## now carry out one boosting iteration
gam1 <- gamboost(y ~ x, control = boost_control(mstop = 0))
lines(x , fitted(gam1), lwd = 3, col = "red")
## add true function
curve((0.5 - 0.9* exp(-50*x^2))*x, add = TRUE, from = -.2,
      to = .2, lty = 4, lwd = 3, col = "blue")

## for a sequence of iterations 
it_values <- c(1, 3, 5, 15,  50, 100, 200, 500, 
               1000,  5000, 50000)
for(j in it_values){
  plot(x, y, las = 1, main = paste("m =", j), pch = 19, xlab = "", ylab = "",
       col = rgb(0.7, 0.7, 0.7, 0.75)) ## observations
  lines(x , fitted(gam1[j]), lwd = 3, col = "red")
  curve((0.5 - 0.9* exp(-50*x^2))*x, add=TRUE, from = -.2,
        to = .2, lty = 4, lwd = 3, col = "blue")
}
dev.off()

###########################################################
##  Determine optimal mstop
###########################################################

## set initial value
mstop(gam1) <- 500

## 25-fold bootstrap
cvr <- cvrisk(gam1)

## results
plot(cvr)

## resulting optimal mstop
mstop(cvr)
