###########################################################
## 
## Analysis of Indian stunting data
## 
###########################################################
##
## Authors of the Code: Benjamin Hofner, Andreas Mayr
## 
## This code accompanies the paper
##
##   Boosting for statistical modelling -- 
##   a non-technical introduction
## 
##   by Andreas Mayr and Benjamin Hofner
## 
###########################################################

## Set seed to make analysis reproducible

set.seed(09052015)


###########################################################
## Install packages if not available
###########################################################

## Install the actual boosting packae
if (!require("mboost")) {
    install.packages("mboost")
}

## The Indian stunting data is contained in this package
if (!require("gamboostLSS")) {
    install.packages("gamboostLSS")   
}

## In order to plot and pre-process the spatial data (based on Indian regions)
## we need to install R2BayesX
if (!require("R2BayesX")) {
    install.packages("R2BayesX")   
}

###########################################################
## Load all required packages and data set
###########################################################

library("mboost")
library("gamboostLSS")
library("R2BayesX")

## Load data sets
data("india", package = "gamboostLSS")
data("india.bnd", package = "gamboostLSS")
## See ?india for details on the data set

###########################################################
## Inspect the data set
###########################################################

## First, let's have a look at the spatial distribution
par(mar = c(1, 0, 2, 0))
india_agg <- data.frame(mcdist = names(tapply(india$stunting, india$mcdist, mean, na.rm = TRUE)),
                        stunting = tapply(india$stunting, india$mcdist, mean, na.rm = TRUE))
plotmap(india.bnd, india_agg, pos = "bottomright",
        range = c(-5.1, 1.50), mar.min = NULL, col = heat.colors(length(india.bnd)))
dev.off()

###########################################################
## Fit and tune model
###########################################################

## Compute neighborhood structure of indian states
neighborhood <- bnd2gra(india.bnd)

## Fit the model:
## - Use smooth effects for mother's age, mother's BMI, child's age and child's BMI
## - Use a Markov random field for the spatial information which is based on Indian districts
## - We fit a Gaussian regression model ("least squares regression")
model1 <- gamboost(stunting ~ bbs(mage) + bbs(mbmi) + bbs(cage)
                      + bmrf(mcdist, bnd = neighborhood),
                      data = india, family = Gaussian())


## Use cross-validation to find the optimal boosting iteratuion.
## Here, we use 25-fold subsampling (with subsamples of size n/2).
folds <- cv(model.weights(model1), type = "subsampling", B = 25)
cvr <- cvrisk(model1, grid = 1:2000, folds = folds)

## Plot the out-of-bag risk for each of the 25 models
plot(cvr)

## Extract the optimal boosting iteration (i.e., the iteration with the minimal average risk)
mstop(cvr)

## Use the optimal number of iterations for the original model
## - this can be simply done by setting mstop of that model to a different value
mstop(model1) <- mstop(cvr)

###########################################################
## Plot and interpret resulting model
###########################################################

## Plot smooth effects
par(mfrow = c(2, 2))
plot(model1, which = "mage", type = "l")
plot(model1, which = "mbmi", type = "l")
plot(model1, which = "cage", type = "l")
#plot(model1, which = "cbmi", type = "l")
dev.off()

## Plot the spatial map; This is a bit more difficult for Markov random fields:
fitted_spatial <- tapply(fitted(model1, which = "mcdist"), india$mcdist, FUN = mean)
plotdata <- data.frame(region = names(fitted_spatial), x = fitted_spatial)
plotmap(india.bnd, plotdata, range = c(-0.66, 0.87),
        pos = "bottomright", mar.min = NULL)


###########################################################
## Reproduction of figures in the manuscript
###########################################################

## Spatial pattern
pdf("Figures/India_spatial_raw.pdf", width = 5, height = 5)
par(mar = c(1, 0, 2, 0))
india_agg <- data.frame(mcdist = names(tapply(india$stunting, india$mcdist, mean, na.rm = TRUE)),
                        stunting = tapply(india$stunting, india$mcdist, mean, na.rm = TRUE))
plotmap(india.bnd, india_agg, pos = "bottomright",
        range = c(-5.1, 1.50), mar.min = NULL, col = heat.colors(length(india.bnd)))
dev.off()


## Smooth effects of the model

## Plot smooth effects
pdf("../Figures/India_smooth.pdf", height = 5, width = 7)
par(mfrow = c(2, 2), mar = c(5, 4.5, 0.5, 0.5))
plot(model1, which = "mage", type = "l",
     xlab = "Age (mother; years)")
plot(model1, which = "mbmi", type = "l",
     xlab = "BMI (mother)")
plot(model1, which = "cage", type = "l",
     xlab = "Age (child; months)")
#plot(model1, which = "cbmi", type = "l",
#     xlab = "BMI (child)")
dev.off()

pdf("../Figures/India_spatial.pdf", width = 5, height = 5)
par(mar = c(1, 0, 2, 0))
fitted_spatial <- tapply(fitted(model1, which = "mcdist"), india$mcdist, FUN = mean)
plotdata <- data.frame(region = names(fitted_spatial), x = fitted_spatial)
plotmap(india.bnd, plotdata, range = c(-0.66, 0.87),
        pos = "bottomright", mar.min = NULL, col = heat.colors(length(india.bnd)))
dev.off()
