###########################################################
## 
## Analysis of breast cancer data
## 
###########################################################
##
## Authors of the Code: Benjamin Hofner, Andreas Mayr
## 
## This code accompanies the paper
##
##   Boosting for statistical modelling -- 
##   a non-technical introduction
## 
##   by Andreas Mayr and Benjamin Hofner
## 
###########################################################

## Set seed to make analysis reproducible

set.seed(09052015)


###########################################################
## Install packages if not available
###########################################################

## Install the actual boosting packae
if (!require("mboost")) {
    install.packages("mboost")
}

## Needed for the installation of the data package
if (!require("devtools")) {
    install.packages("devtools")
}

## The gene expression data is contained in this package; 
## note that the download takes a while
if (!require("datamicroarray")) {
    library("devtools")
    install_github("boost-R/datamicroarray")
}

## To further evaluate the model we use the ROCR package
if (!require("ROCR")) {
    install.packages("ROCR")
}

###########################################################
## Load all required packages and data set
###########################################################

library("mboost")
library("datamicroarray")

data("gravier", package = "datamicroarray")

## The data set is stored as a list:
str(gravier)

## Store names of x
nms <- names(gravier$x)

## Reorder (y as first element; only for "cosmetic reasons")
gravier <- gravier[2:1]

## Coerce to data frame
gravier <- as.data.frame(gravier)

## Reset names
names(gravier)[-1] <- nms
gravier[1:10, 1:10]

###########################################################
## Fit and tune model
###########################################################

## Fit the initial model:
## - linear model with outcome y and all other variables as predictores (".")
## - data set is the (modified) gravier data
## - we fit a logistic regression model
model2 <- glmboost(y ~ ., data = gravier, family = Binomial())

## Optimize / tune the number of boosting iterations:
## - per default we use 25-fold bootstrap
## - we use the model and optimize the out-of-bag risk
cvr <- cvrisk(model2)

## Plot the out-of-bag risk for each of the 25 models
plot(cvr)

## Determine the optimal boosting iteration (i.e., the iteration with the minimal average risk)
mstop(cvr)

## Plot the coefficient paths (i.e., the evolution of the coefficients)
par(mfrow = c(2, 1), mar = c(5, 4, 0.1, 6))
plot(model2, main = "")
abline(v = mstop(cvr), lwd = 1, lty = 2)

## Use the optimal number of iterations for the original model
## - this can be simply done by setting mstop of that model to a different value
mstop(model2) <- mstop(cvr)

## Plot the path for until the optimal boosting iteration
plot(model2, main = "")

## Extract the coefficients of the optimal model
coef(model2)

## One can see, that in this specific model (logit model) we need to multiply 
## the coefficients by 2. Additionally, we have a spearate offset (which is the
## constant initialization value of the algorithm). 
## - to add the offset to the other coefficients one can use the option off2int = TRUE
## - additionally we multiply the coefficients by 2
## Thus we get "standard" regression coefficients:
2 * coef(model2, off2int = TRUE)

## Overall, we selected 
length(coef(model2)) - 1
## out of 
ncol(gravier) - 1
  ## predictor variables.

## In order to make predictions (here for the first data set) for the
## probability of a poor outcome (which is the second category in this data set),
## one can use
predict(model2, newdata = gravier[1, ], type = "response")


###########################################################
## Further evaluate the model
###########################################################

library("ROCR")

## The ROC on the training data looks as follows
par(mfrow = c(1, 1), mar = c(5, 4, 0.1, 0.1))
preds <- prediction(predict(model2, type = "response"), labels = gravier$y)
plot(performance(preds, "tpr", "fpr"), lwd = 2)
abline(0, 1, col = "grey")

## and the corresponding AUC is
(AUC <- performance(preds, "auc")@y.values[[1]])

## Of note, this is a bit to optimistic as we have tuned and validated the model
## on the same data set. However, for the purpose of a simple and straight forward
## example we consider this acceptable.

## The optimal cutoff on this data would be
opt.cut = function(perf, pred){
    cut.ind = mapply(FUN=function(x, y, p){
        d = (x - 0)^2 + (y-1)^2
        ind = which(d == min(d))
        c(sensitivity = y[[ind]], specificity = 1 - x[[ind]], 
          cutoff = p[[ind]])
    }, perf@x.values, perf@y.values, pred@cutoffs)
    return(cut.ind)
}
(optimal_cutoff <- opt.cut(performance(preds, "tpr", "fpr"), preds)[3])
## i.e. if the predicted probability for poor prognosis exeeds 0.38, we classify
## the patient as "poor".

###########################################################
## Redo analysis with split data set (learning and test set)
###########################################################

## Use random samples of the data to learn (n = 100) and test the model (n = 68)
idx_learn <- sample(1:nrow(gravier), 100)
learn <- gravier[idx_learn, ]
test <- gravier[-idx_learn, ]

## Fit the initial model:
model2_learn <- glmboost(y ~ ., data = learn, family = Binomial())

## Optimize / tune the number of boosting iterations:
cvr_learn <- cvrisk(model2_learn)

## Plot the out-of-bag risk for each of the 25 models
plot(cvr_learn)

## Use the optimal number of iterations for the original model
mstop(model2_learn) <- mstop(cvr_learn)

## Predict outcome for test set
predict(model2_learn, newdata = test, type = "response")

## Compute AUC and plot ROC curve for new data
par(mfrow = c(1, 1), mar = c(5, 4, 0.1, 0.1))
preds_test <- prediction(predict(model2, newdata = test, type = "response"), labels = test$y)
plot(performance(preds_test, "tpr", "fpr"), lwd = 2)
abline(0, 1, col = "grey")
(AUC_test <- performance(preds_test, "auc")@y.values[[1]])

## Of note, we could also repeatedly split the data into learn and test data to 
## further evaluate the sensitivity of the prediction performance with respect
## to the data

## To see how much the model on the whole data set and on the learn data set
## have in common, we finally lok at the intercetion
intersect(names(coef(model2)), names(coef(model2_learn)))
## and differences of selected base-learners
setdiff(names(coef(model2)), names(coef(model2_learn)))
setdiff(names(coef(model2_learn)), names(coef(model2)))

###########################################################
## Reproduction of figures in the manuscript
###########################################################

pdf("Figures/cvrisk.pdf", width = 7, height = 5)
plot(cvr)
dev.off()

mstop(model2) <- 4
pdf("../Figures/coefficient_path.pdf", width = 7, height = 3.5)
par(mar = c(5, 4, 0.1, 6))
plot(model2, main = "", off2int = TRUE, xaxt = "n", yaxt = "n", 
     xlim = c(0,4))
## use pretty x-axis (only needed as we have so little iterations)
axis(1, at = c(0:4))
## multiply y-axis by two (see warning)
axis(2, at = seq(-0.6, 0.4, by = 0.2), round(seq(-1.2, 0.8, by = 0.4), 2))

# for better illustration
# include coefs at mstop = 0 
lines(x = c(0,1), y = c(model2$offset,coef(model2[1])[1] + model2$offset), col = "#FF0000" )
lines(x = c(0,1), y = c(0,coef(model2[1])[2]), col = "#FF0000" )
lines(x = c(0,1), y = c(0,0), col = "#FF0000" )
dev.off()

mstop(model2) <- mstop(cvr)
pdf("Figures/ROC.pdf", width = 3.5, height = 3.5)
par(mfrow = c(1, 1), mar = c(5, 4, 0.1, 0.1))
preds <- prediction(predict(model2, type = "response"), labels = gravier$y)
plot(performance(preds, "tpr", "fpr"), lwd = 2)
abline(0, 1, col = "grey")
dev.off()

pdf("Figures/ROC_test.pdf", width = 3.5, height = 3.5)
par(mfrow = c(1, 1), mar = c(5, 4, 0.1, 0.1))
preds_test <- prediction(predict(model2, newdata = test, type = "response"), labels = test$y)
plot(performance(preds_test, "tpr", "fpr"), lwd = 2)
abline(0, 1, col = "grey")
dev.off()

## coefficient table
## To create a LaTeX table, we use xtable
if (!require("xtable")) {
    install.packages("xtable")
}
library("xtable")
all_selected_vars <- unique(c(selected(model2), selected(model2_learn)))
tab <- cbind(2 * coef(model2, which = all_selected_vars), 
             2* coef(model2_learn, which = all_selected_vars))
tab[tab == 0] <- NA
colnames(tab) <- c("Full data", "Learning data")
print(xtable(tab, align = "lrr", label = "tab:genes", 
             caption = "Coefficient estiamtes of prognostic genes for full data set and learning data set."), 
      booktabs = TRUE, tabular.environment = "longtable", floating = FALSE)

